public class OrderedLinkedList {

    public static final int ASC  = -1;
    public static final int DESC = 1;

    int                     order;
    int                     size;
    ListNode                head, tail;

    public OrderedLinkedList( int order )
    {
        this.order = order;

        size = 0;
        head = tail = null;
    }

    public OrderedLinkedList()
    {
        this( ASC );
    }

    public int getOrder()
    {
        return order;
    }

    public void insert( Comparable value )
    {
        ++size;
        ListNode newNode = new ListNode( value );

        // empty list
        if( head == null )
        {
            head = tail = newNode;
            return;
        }

        // insert at the beginning, where
        // head > value and ASC, or
        // head < value and DESC
        if( head.value.compareTo( value ) * order < 0 )
        {
            newNode.next = head;
            head.prev = newNode;
            head = newNode;
            return;
        }

        // insert at the end, where
        // tail < value and ASC, or
        // tail > value and DESC
        if( value.compareTo( tail.value ) * order < 0 )
        {
            newNode.prev = tail;
            tail.next = newNode;
            tail = newNode;
            return;
        }

        // insert in the middle
        ListNode previous, current;
        previous = current = head;
        while( current != null )
        {
            if( value.compareTo( current.value ) * order < 0 )
            {
                previous = current;
                current = current.next;
            }
            else
            {
                previous.next = newNode;
                current.prev = newNode;
                newNode.next = current;
                newNode.prev = previous;
                return;
            }
        }
    }

    public OrderedLinkedList intersect( OrderedLinkedList other )
    {
        // To Be Implemented
        return null;
    }

    public void insert( OrderedLinkedList other )
    {
        // To Be Implemented
    }

    public OrderedLinkedList remove( int beginIndex, int endIndex )
    {
        // To Be Implemented
        return null;
    }

    public void reverse()
    {
        ListNode tmp, current = tail;

        while( current != null )
        {
            // swap current.next and current.prev
            tmp = current.next;
            current.next = current.prev;
            current.prev = tmp;

            current = current.next;
        }

        // swap head and tail
        tmp = tail;
        tail = head;
        head = tmp;

        order *= -1;
    }

    public int size()
    {
        return size;
    }

    public void print()
    {
        ListNode current = head;
        while( current != null )
        {
            System.out.print( current.value + " " );
            current = current.next;
        }
        System.out.println();
    }

    public static void main( String[] args )
    {
        OrderedLinkedList list1 = new OrderedLinkedList();
        list1.insert( 11 );
        list1.insert( 20 );
        list1.insert( 15 );
        list1.insert( 10 );
        System.out.println( list1.size() );
        list1.print();

        OrderedLinkedList list2 = new OrderedLinkedList( OrderedLinkedList.DESC );
        list2.insert( 20 );
        list2.insert( 14 );
        list2.insert( 25 );
        list2.insert( 12 );
        System.out.println( list2.size() );
        list2.print();
/*
        OrderedLinkedList list3 = list1.intersect( list2 );
        list3.print();

        OrderedLinkedList sublist;

        sublist = list1.remove( 1, 3 );
        sublist.print();
        list1.print();
        list1.insert( sublist );
        list1.print();

        sublist = list1.remove( 0, 3 );
        sublist.print();
        list1.print();
        list1.insert( sublist );
        list1.print();

        sublist = list1.remove( 1, 4 );
        sublist.print();
        list1.print();
        list1.insert( sublist );
        list1.print();

        sublist = list1.remove( 0, 4 );
        sublist.print();
        list1.print();
        list1.insert( sublist );
        list1.print();
*/
    }
}

class ListNode {

    Comparable value;
    ListNode   next, prev;

    ListNode( Comparable value, ListNode next, ListNode prev )
    {
        this.value = value;
        this.next = next;
        this.prev = prev;
    }

    ListNode( Comparable value )
    {
        this( value, null, null );
    }

}
